package conversions

import data.{Cons, Nil}
import org.scalatest.matchers.should.Matchers
import org.scalatest.wordspec.AnyWordSpecLike

import java.io.ByteArrayOutputStream

class LispListSpec extends AnyWordSpecLike with Matchers {

  "LispList" can {

    /**
      * Exercise: Implicit Conversions
      */
    "convert types" which {
      val lispList = Cons(1, Cons(2, Cons(3, Nil)))
      def lengthAsList[A](list: List[A]): Int = list.length
      def lengthAsSeq[A](seq: Seq[A]): Int = seq.length

      "become a List" in {
        pending
        //lengthAsList(lispList) should eq(3)
      }

      "become a Seq" in {
        pending
        //lengthAsSeq(lispList) should eq(3)
      }

      "throw an exception if getting length as a Seq but actually is a List" in {
        pending
        //lengthAsSeq(lispList) should eq(3)
      }

      "become a Seq, but loudly" in {
        val out = new ByteArrayOutputStream()
        Console.withOut(out) {
          pending
          //lengthAsSeq(lispList) should eq(3)
        }
        out.toString should contain("THREE OF THEM!")
      }
    }

    /**
      * Exercise: Explicit conversions with extension methods
      */
    "explicitly convert types" which {
      val lispList = Cons(1, Cons(2, Cons(3, Nil)))

      "asList" can {
        "explicitly become a List" in {
          pending
          //lispList.asList should eq(List(1, 2, 3))
        }
      }
      "asSeq" can {
        "explicitly become a Seq" in {
          pending
          //lispList.asList should eq(Seq(1, 2, 3))
        }
      }
    }
  }
}
